/* ========================================
 *
 * Copyright CONSULTRON, 2020
 * All Rights Reserved
 *
 * CONSULTRON allows PUBLIC use of the file.
 * CONSULTRON provides NO WARRANTY expressed or implied.
 *
 * This file is the functions that parse the commands.  This is 'generic' and could be common across projects.
 * ========================================
Module: `$INSTANCE_NAME`.c
Module Build: `$CY_MAJOR_VERSION`.`$CY_MINOR_VERSION`.4
Module Comment: `$CY_COMMENT`
•	3.3.4 - Increased MENU_LEVEL_MAX to 4
*/

#if !defined(`$INSTANCE_NAME`_H)
#define `$INSTANCE_NAME`_H
	
typedef unsigned char uint8;
typedef unsigned char uint8_t;
	
#define TRUE 1
#define FALSE 0
	
// menu_flags
#define MENU_FUNC	 		0b00000000		// This is a command function
#define MENU_HELP 			0b00010000		// the function is a help print
#define MENU_HELP_ALWAYS 	0b00100000		// always print the help function even with the 'h' command
#define MENU_HIDDEN 		0b01000000		// function/menu is hidden.  Command available but not printed in help menus
// Protection bits in lower nibble (if used) 
#define MENU_PW_MASK		0b00001111		//  Up to 4 protection levels supports
#define MENU_PW_PROT0 		(1<<0)			// function/menu is password protected
#define MENU_PW_PROT1		(1<<1)			// function/menu is password protected
#define MENU_PW_PROT2		(1<<2)			// function/menu is password protected
#define MENU_PW_PROT3		(1<<3)			// function/menu is password protected
#define MENU_LEVEL_MAX			4u		// Maximum menu level supported.  Increase this number with caution.

/*  macros */
#define MENU_CMD_PROTO(func_name) void func_name(char c)
#define MENU_END_ALLOC				{eCmdEnd, "", NULL,MENU_FUNC}		// Keep this "NULL" element as the last element.  This element is used to signal the end of the menu list
#define MENU_FUNC_ALLOC(cmdchar, text, func, flags)		{cmdchar, text, &func, MENU_FUNC|flags}  // macro to simplify Functions
#define MENU_HELP_ALLOC(cmdchar, text, func, flags)		{cmdchar, text, (void *)func, MENU_HELP|flags}  // macro to simplify SubMenus
	
#if defined(UART_SCB__HW)
#define NO_RX_AVAIL ((char)CY_SCB_UART_RX_NO_DATA)
#else
#define NO_RX_AVAIL ((char)'\0')
#endif


/* Here is the enumerator for the commands. 
	Each enumerated var should be a 'printable' ascii char.  (Exception is the eCmdEnd which must be present as the very last cmd in the menu_cmd array.)
	Order in the enum below is not important.
	Make sure there are no duplicates.  There are no checks for duplicates.  In the case of a duplicate the first match will be executed.
*/
typedef enum
{
eCmdHelpMax = '?',
eCmdHelpMin = 'h',
eCmdBuildInfo = 'b',

eCmdEnd = '\0',	//This element is used to signal the end of the menu list
} menu_std_chars_t;
	
typedef struct
{
	char menu_char;	// mode character(s)
	char *menu_text;	// mode text
	void (*menu_func)(char);		// menu function optional argument(s) to pass
	uint8 menu_flags;				// function menu_flags listed above
}menu_cmd;


/* Global vars
*/
extern const menu_cmd menu_array_L1[];
extern uint8_t pw_locks;		// PW lock switch; TRUE = locked (default)

/* Function Prototypes
*/
int `$INSTANCE_NAME`_SetCallbacks(void (*putstring)(char *string));
// For now, do not use _Init and _Start from the Term Component
#define  `$INSTANCE_NAME`_Init()		`$INSTANCE_NAME`_SetCallbacks((void *)&`$Term_name`_PutString)	// direct function redirect
#define  `$INSTANCE_NAME`_Start()		`$INSTANCE_NAME`_Init()	// direct function redirect

_Bool `$INSTANCE_NAME`_process(const menu_cmd menu_array[], uint8_t c);
int `$INSTANCE_NAME`_num_func_SetCallback(void (*num_func_cb));
const menu_cmd *`$INSTANCE_NAME`_search(const menu_cmd menu_array[], uint8_t c);

#endif	// `$INSTANCE_NAME`_H
/* [] END OF FILE */
