/* ========================================
  *
 * Copyright CONSULTRON, 2020
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONSULTRON allows PUBLIC use of the file.
 * CONSULTRON provides NO WARRANTY expressed or implied.
 *
* ========================================
Module: `$INSTANCE_NAME`.c
Module Build: `$CY_MAJOR_VERSION`.`$CY_MINOR_VERSION`.4
Module Comment: `$CY_COMMENT`

•	2.0.3 - GetString_Filt() no longer increments the char count with '\r'.
•	2.0.4 - Support of CR ('\r') and LF ('\n) in systems that support either as the ENTER key character.
*/
/***********************************************
The file is used to declare the GetString() chars used and the filtering type allowed for each char.
*/
#include "`$INSTANCE_NAME`.h"

char ascii_chars[] = 
{
/*Hex char  description 						  */
/*20	 	Space								  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*21	!	Exclamation mark					  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*22	"	Double quotes (or speech marks)		  */	GS_ALPHA_PNC,
/*23	#	Number								  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*24	$	Dollar								  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*25	%	Per cent sign						  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*26	&	Ampersand							  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*27	'	Single quote						  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*28	(	Open parenthesis (or open bracket)	  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*29	)	Close parenthesis (or close bracket)  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*2A	*	Asterisk							  */	GS_ALPHA_PNC,
/*2B	+	Plus								  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC | GS_NUM_SIGN,
/*2C	,	Comma								  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*2D	-	Hyphen								  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC | GS_NUM_SIGN,
/*2E	.	Period, dot or full stop			  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC | GS_NUM_DP,
/*2F	/	Slash or divide						  */	GS_ALPHA_PNC,
/*30	0	Zero								  */	GS_NUM,
/*31	1	One									  */	GS_NUM,
/*32	2	Two									  */	GS_NUM,
/*33	3	Three								  */	GS_NUM,
/*34	4	Four								  */	GS_NUM,
/*35	5	Five								  */	GS_NUM,
/*36	6	Six									  */	GS_NUM,
/*37	7	Seven								  */	GS_NUM,
/*38	8	Eight								  */	GS_NUM,
/*39	9	Nine								  */	GS_NUM,
/*3A	:	Colon								  */	GS_ALPHA_PNC,
/*3B	;	Semicolon							  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*3C	<	Less than (or open angled bracket)	  */	GS_ALPHA_PNC,
/*3D	=	Equals								  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*3E	>	Greater than (or close angled bracket)*/	GS_ALPHA_PNC,
/*3F	?	Question mark						  */	GS_ALPHA_PNC,
/*40	@	At symbol							  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*41	A	Uppercase A							  */	GS_ALPHA | GS_NUM_HEX,
/*42	B	Uppercase B							  */	GS_ALPHA | GS_NUM_HEX,
/*43	C	Uppercase C							  */	GS_ALPHA | GS_NUM_HEX,
/*44	D	Uppercase D							  */	GS_ALPHA | GS_NUM_HEX,
/*45	E	Uppercase E							  */	GS_ALPHA | GS_NUM_HEX | GS_NUM_EXP,
/*46	F	Uppercase F							  */	GS_ALPHA | GS_NUM_HEX,
/*47	G	Uppercase G							  */	GS_ALPHA,
/*48	H	Uppercase H							  */	GS_ALPHA,
/*49	I	Uppercase I							  */	GS_ALPHA,
/*4A	J	Uppercase J							  */	GS_ALPHA,
/*4B	K	Uppercase K							  */	GS_ALPHA,
/*4C	L	Uppercase L							  */	GS_ALPHA,
/*4D	M	Uppercase M							  */	GS_ALPHA,
/*4E	N	Uppercase N							  */	GS_ALPHA,
/*4F	O	Uppercase O							  */	GS_ALPHA,
/*50	P	Uppercase P							  */	GS_ALPHA,
/*51	Q	Uppercase Q							  */	GS_ALPHA,
/*52	R	Uppercase R							  */	GS_ALPHA,
/*53	S	Uppercase S							  */	GS_ALPHA,
/*54	T	Uppercase T							  */	GS_ALPHA,
/*55	U	Uppercase U							  */	GS_ALPHA,
/*56	V	Uppercase V							  */	GS_ALPHA,
/*57	W	Uppercase W							  */	GS_ALPHA,
/*58	X	Uppercase X							  */	GS_ALPHA,
/*59	Y	Uppercase Y							  */	GS_ALPHA,
/*5A	Z	Uppercase Z							  */	GS_ALPHA,
/*5B	[	Opening bracket						  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*5C	\	Backslash							  */	GS_ALPHA_PNC,
/*5D	]	Closing bracket						  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*5E	^	Caret - circumflex					  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*5F	_	Underscore							  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*60	`	Grave accent						  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*61	a	Lowercase a							  */	GS_ALPHA | GS_NUM_HEX,
/*62	b	Lowercase b							  */	GS_ALPHA | GS_NUM_HEX,
/*63	c	Lowercase c							  */	GS_ALPHA | GS_NUM_HEX,
/*64	d	Lowercase d							  */	GS_ALPHA | GS_NUM_HEX,
/*65	e	Lowercase e							  */	GS_ALPHA | GS_NUM_HEX | GS_NUM_EXP,
/*66	f	Lowercase f							  */	GS_ALPHA | GS_NUM_HEX,
/*67	g	Lowercase g							  */	GS_ALPHA,
/*68	h	Lowercase h							  */	GS_ALPHA,
/*69	i	Lowercase i							  */	GS_ALPHA,
/*6A	j	Lowercase j							  */	GS_ALPHA,
/*6B	k	Lowercase k							  */	GS_ALPHA,
/*6C	l	Lowercase l							  */	GS_ALPHA,
/*6D	m	Lowercase m							  */	GS_ALPHA,
/*6E	n	Lowercase n							  */	GS_ALPHA,
/*6F	o	Lowercase o							  */	GS_ALPHA,
/*70	p	Lowercase p							  */	GS_ALPHA,
/*71	q	Lowercase q							  */	GS_ALPHA,
/*72	r	Lowercase r							  */	GS_ALPHA,
/*73	s	Lowercase s							  */	GS_ALPHA,
/*74	t	Lowercase t							  */	GS_ALPHA,
/*75	u	Lowercase u							  */	GS_ALPHA,
/*76	v	Lowercase v							  */	GS_ALPHA,
/*77	w	Lowercase w							  */	GS_ALPHA,
/*78	x	Lowercase x							  */	GS_ALPHA,
/*79	y	Lowercase y							  */	GS_ALPHA,
/*7A	z	Lowercase z							  */	GS_ALPHA,
/*7B	{	Opening brace						  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*7C	|	Vertical bar						  */	GS_ALPHA_PNC,
/*7D	}	Closing brace						  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
/*7E	~	Equivalency sign - tilde			  */	GS_ALPHA_PNC | GS_ALPHA_FS_PNC,
};

/*************************************************************************************************
Functions for parsing input
*/
#define ALPHA_CHAR_START 0x20	// SPACE
#define ALPHA_CHAR_END 0x7E		// '~'

uint8_t GetChar_def(void) { return('\0'); } // default function if not set by the user.  (To prevent CPU crash).
void PutChar_def(char c) { c=c; return; } // default function if not set by the user.  (To prevent CPU crash).

// callback function 
uint8_t (*GetChar_callback)(void)   = &GetChar_def;	// callback pointer to the num_func() if needed.
void (*PutChar_callback)(char c) = &PutChar_def;	// callback pointer to the PutString()

/*************************************************************************************************
int `$INSTANCE_NAME`_SetCallbacks( char (*GetChar_cb)(void), void (*PutChar_cb)(char pc_c))
**************************************************************************************************

This is a function to register a callback function at run-time.
This is used in Get_Char_Filt() and Get_String_Filt(). 
This MUST BE set with valid pointers before using the commands mentioned above otherwise:
- NO terminal display will occur.
- No strings will be processed.

Note:  `$INSTANCE_NAME`_Init() or `$INSTANCE_NAME`_Start() will set these callbacks
	if you provide a valid instanced name of your Terminal (UART-type) component.
	In this case, this function does need to be called. 
	However, you can use this function at run-time to change Terminal communication
	ports on-the-fly.

	Parameters:
		GetChar_cb = callback pointer to the terminal GetChar() function
		PutChar_cb = callback pointer to the terminal PutChar() function

	Returns:
		0 = No error (For now)
*/
int `$INSTANCE_NAME`_SetCallbacks( uint8_t (*GetChar_cb)(void), void (*PutChar_cb)(char pc_c))
{
	GetChar_callback = GetChar_cb;
	PutChar_callback = PutChar_cb;	// set the run-time callback
	return(0);
}

/*************************************************************************************************
* uint8_t `$INSTANCE_NAME`_GetChar_Filtcc() - Get string from comm device with input filter with control chars
**************************************************************************************************
*
* Parameters:
*  filter: enum that defines input characters to filter
*		filter types supported: 
*					unsigned decimal fixed (eq => 134.456)
*					signed decimal fixed (eq => -134.456 or +35.7)
*					unsigned decimal integer (eq => 23045)
*					signed decimal integer (eq => -23045 or +45)
*					hexidecimal integer (without the 0x or $ to precede it, eq => 8AD34f5)
*					string (any input character allowed.)
*					Filesystem string (any input character allowed except disallowed chars.  eg => This is a test_1.new#)
*	allow_cc: Boolean. TRUE = pass \b or \r back to the calling routine.  FALSE = ignore \b or pass NULL in place of \r
*
* Return:
*  a filtered single character.
*
* Notes:
* Control Characters:
* Supported:
* 	BACKSPACE ('\b') is supported to delete previously inputted characters.
*	Carriage Return ('\r') is used to end the GetString session.  It is substituted with NULL ('\0') to end the string.
* Not supported: 
*	DELETE and right-arrow and left-arrow keys.
*
*/
uint8_t `$INSTANCE_NAME`_GetChar_Filtcc(input_filter_types filter, _Bool allow_cc)
{
char new_c = '\0';
	while(1)		// loop until a valid char is selected that matches the filter and kicks out with a 'break'
	{
		while ((new_c = GetChar_callback()) == '\0')	{}	// Waits for non-NULL char[blocking]
		{
			switch(new_c)	// detect backspace
			{
			case ALPHA_CHAR_START ... ALPHA_CHAR_END:		// Alpha chars include punctuations.
				if(ascii_chars[new_c-ALPHA_CHAR_START] & filter)
				{	// Only of the character chosen matches ANY of the filters
					PutChar_callback(new_c);	// echo the char except for BACKSPACE
				}
				else new_c = '\0';		// filter out this char.  Clear it for the return.
				break;
			// Special cases for control chars.
			case '\b':	// backspace (No echo)
				break;
			case '\r':	// carriage return for ENTER key
				if(!allow_cc) new_c = '\0'; 	 //  control char not allowed: pass NUL
				PutChar_callback(new_c);	// echo the char
				PutChar_callback('\n');	// echo the char in case the Terminal program uses this as the line advance.
				break;
			case '\n':	// line feed for ENTER key
				if(!allow_cc) new_c = '\0'; 	 //  control char not allowed: pass NUL
				PutChar_callback(new_c);	// echo the char
				PutChar_callback('\r');	//  echo the char in case the Terminal program uses this as the line advance.
				break;
			default:						// everthing else filtered out.
				new_c = '\0';		// filter out this char.  Clear it for the return.
				break;
			}
			break;	// break out of while(1) loop
		}
	}
	return(new_c);
}

/*************************************************************************************************
* uint8_t `$INSTANCE_NAME`_GetString_Filt() - Get string from comm device with input filter
**************************************************************************************************
*
* Parameters:
*  string[]: Pointer to the string array residing in RAM
*  sizeof(string): size of the string array residing in RAM.  This is used to check to make sure we don't exceed the size of the string available.
*  filter: enum that defines input characters to filter
*		filter types supported: 
*					unsigned decimal fixed (eq => 134.456)
*					signed decimal fixed (eq => -134.456 or +35.7)
*					unsigned decimal integer (eq => 23045)
*					signed decimal integer (eq => -23045 or +45)
*					hexidecimal integer (without the 0x or $ to precede it, eq => 8AD34f5)
*					string (any input character allowed.)
*					Filesystem string (any input character allowed except disallowed chars.  eg => This is a test_1.new#)
*
* Return:
*  string length
*
* Notes:
* Control Characters:
* Supported:
* 	BACKSPACE ('\b') is supported to delete previously inputted characters.
*	Carriage Return ('\r') is used to end the GetString session.  It is substituted with NULL ('\0') to end the string.
* Not supported: 
*	DELETE and right-arrow and left-arrow keys.
*
*/
uint8_t `$INSTANCE_NAME`_GetString_Filt(char string[], uint16_t string_sz, input_filter_types filter)
{
uint8 i=0;
char new_c = '\0';
_Bool send_store = FALSE;
	while(new_c != '\r')		// Carriage return is the exit from the while()
	{
		send_store = FALSE;		// reset send and store flag
		while((new_c = `$INSTANCE_NAME`_GetChar_Filtcc(filter, TRUE)) == '\0')	{}	// Waits for non-NULL char[blocking]
		if(i > string_sz-1) {}	// makes sure the input doesn't exceed the string size.
		else
		{
			switch(new_c)	// detect backspace
			{
			// Special cases for control chars.
			case '\b':	// backspace
				// special case for backspace
				if(i > 0) i--;	// found backspace.  Do not put on returned string and remove previous char.
				break;
			case '\r':	// carriage return
			default:						// pre=filterd by Term_GetString_Filt()
				send_store = TRUE; // filter satisfied. enter character in string
				break;
			}

			if(send_store)
			{
				if(new_c == '\r') string[i] = '\0';	// special case for the carriage return
				else string[i++] = new_c;	// filter satisfied. enter character in string
			}
		}
	}
	return(i);
}


/* [] END OF FILE */
