/* ========================================
 *
 * Copyright CONSULTRON, 2020
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONSULTRON allows PUBLIC use of the file.
 * CONSULTRON provides NO WARRANTY expressed or implied.
 *
 * This file is the commands and functions specific to the project.
 * ========================================

Module: `$INSTANCE_NAME`.c
Module Build: `$CY_MAJOR_VERSION`.`$CY_MINOR_VERSION`.4
Module Comment: `$CY_COMMENT`
*/

#if !defined(`$INSTANCE_NAME`_H)
#define `$INSTANCE_NAME`_H
	
typedef unsigned char uint8;
typedef unsigned char uint8_t;
typedef unsigned short uint16;
typedef unsigned short uint16_t;
	
/* Here is the enumerator for the commands. 
	Each enumerated var should be a 'printable' ascii char.  (Exception is the eCmdEnd which must be present as the very last cmd in the menu_cmd array.)
	Order in the enum below is not important.
	Make sure there are no duplicates.  There are no checks for duplicates.  In the case of a duplicate the first match will be executed.
*/
	
#define TRUE 1
#define FALSE 0

/* Bit defines for the GetString validation of the ascii character set. */
#define GS_NUM			(0b00000001)		// Number flag (0 1 2 3 4 5 6 7 8 9 allowed)
#define GS_NUM_DP		(0b00000010)		// Decimal point flag (. allowed)
#define GS_NUM_SIGN		(0b00000100)		// Signed flag	(+ - allowed) 
#define GS_NUM_EXP		(0b00001000)		// Exponent flag (e E allowed)
#define GS_NUM_HEX		(0b00010000)		// Hexidecimal flag (a b c d e f A B C D E F allowed)
#define GS_ALPHA		(0b00100000)		// Alphabet flag (a - z  A - Z allowed)
#define GS_ALPHA_PNC	(0b01000000)		// Punctuation flag (SPACE ! " # $ % & ' ( ) * + , - . / : ; < = > ? @ [ \ ] ^ _ ` { | } ~  allowed)
#define GS_ALPHA_FS_PNC	(0b10000000)		// File System Punctuation flag (SPACE ! " # $ % & ' ( ) * + , - . ; = @ [ ] ^ _ ` { } ~  allowed)

typedef enum
{
	ift_string = GS_ALPHA | GS_NUM | GS_ALPHA_PNC,				// string chars
	ift_dec = GS_NUM,											// decimal (integer) chars (no sign or decimal point)
	ift_sdec = GS_NUM | GS_NUM_SIGN,							// signed decimal (integer) chars (no decimal point)
	ift_dec_fixed = GS_NUM | GS_NUM_DP,							// decimal fixed-point chars (no sign)
	ift_sdec_fixed = GS_NUM | GS_NUM_DP | GS_NUM_SIGN,			// signed decimal fixed-point chars
	ift_hex = GS_NUM | GS_NUM_HEX,								// unsigned hexadecimal chars
	ift_str_fs = GS_ALPHA | GS_NUM | GS_ALPHA_FS_PNC,			// string chars supported in FileSystem naming.
	ift_float = GS_NUM | GS_NUM_DP | GS_NUM_SIGN | GS_NUM_EXP,  // float chars
} input_filter_types;

/* Function Prototypes
*/
int `$INSTANCE_NAME`_SetCallbacks( uint8_t (*GetChar_cb)(void), void (*PutChar_cb)(char pc_c));
// For now, do not use _Init and _Start from the Term Component
#define  `$INSTANCE_NAME`_Init()		`$INSTANCE_NAME`_SetCallbacks((void *)&`$Term_name`_GetChar, (void *)&`$Term_name`_PutChar)	// direct function redirect
#define  `$INSTANCE_NAME`_Start()		`$INSTANCE_NAME`_Init()	// direct function redirect

// There is no need for _Stop().
uint8_t `$INSTANCE_NAME`_GetString_Filt(char string[], uint16_t string_sz, input_filter_types filter);
uint8_t `$INSTANCE_NAME`_GetChar_Filt(input_filter_types filter);
uint8_t `$INSTANCE_NAME`_GetChar_Filtcc(input_filter_types filter, _Bool allow_cc);
#define `$INSTANCE_NAME`_GetChar_Filt(filter) `$INSTANCE_NAME`_GetChar_Filtcc(filter, FALSE)

#endif	// `$INSTANCE_NAME`_H
/* [] END OF FILE */
